@@ -48,35 +48,46 @@ from wavepy2.util.common import common_tools
 from wavepy2.util.common.common_tools import FourierTransform
 from wavepy2.util.log.logger import get_registered_logger_instance, get_registered_secondary_logger, register_secondary_logger, LoggerMode
 from wavepy2.util.plot.plotter import get_registered_plotter_instance
+from wavepy2.util.plot.plot_tools import PlottingProperties
 from wavepy2.util.ini.initializer import get_registered_ini_instance
 from wavepy2.util.io.read_write_file import read_tiff
 
 from wavepy2.tools.common.wavepy_data import WavePyData
 
-from wavepy2.core import grating_interferometry
+from wavepy2.tools.common.bl import grating_interferometry, crop_image
 
-from wavepy2.core.widgets.crop_dialog_widget import CropDialogPlot
-from wavepy2.core.widgets.harmonic_grid_plot_widget import HarmonicGridPlot
-from wavepy2.core.widgets.harmonic_peak_plot_widget import HarmonicPeakPlot
-from wavepy2.tools.common.widgets.colorbar_crop_dialog_widget import ColorbarCropDialogPlot
+from wavepy2.tools.common.widgets.harmonic_grid_plot_widget import HarmonicGridPlot
+from wavepy2.tools.common.widgets.harmonic_peak_plot_widget import HarmonicPeakPlot
 from wavepy2.tools.common.widgets.show_cropped_figure_widget import ShowCroppedFigure
 
-from wavepy2.tools.diagnostic.coherence.widgets.sgz_input_parameters_widget import SGZInputParametersWidget, generate_initialization_parameters_sgz, PATTERNS, ZVEC_FROM
+from wavepy2.tools.diagnostic.coherence.widgets.sgz_input_parameters_widget import SGZInputParametersWidget, SGZInputParametersDialog, generate_initialization_parameters_sgz, PATTERNS, ZVEC_FROM
 from wavepy2.tools.diagnostic.coherence.widgets.visibility_widget import VisibilityPlot
 from wavepy2.tools.diagnostic.coherence.widgets.fit_period_vs_z_widget import FitPeriodVsZPlot
 
 SINGLE_THREAD = 0
-MULTI_TREAD = 1
+MULTI_THREAD = 1
 
 class SingleGratingCoherenceZScanFacade:
-    def get_initialization_parameters(self, script_logger_mode): raise NotImplementedError()
-    def calculate_harmonic_periods(self, initialization_parameters): raise NotImplementedError()
-    def run_calculation(self, harm_periods_result, initialization_parameters): raise NotImplementedError()
-    def sort_calculation_result(self, run_calculation_result, initialization_parameters): raise NotImplementedError()
+    def draw_initialization_parameters_widget(self, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+    def get_initialization_parameters(self, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+    def manager_initialization(self, initialization_parameters, script_logger_mode=LoggerMode.FULL, show_fourier=False): raise NotImplementedError()
 
-def create_single_grating_coherence_z_scan_manager(mode=MULTI_TREAD):
-    return __SingleGratingCoherenceZScanMultiThread() if mode==MULTI_TREAD else __SingleGratingCoherenceZScanSingleThread()
+    def draw_crop_initial_image(self, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+    def crop_initial_image(self, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+    def draw_crop_dark_image(self, initial_crop_parameters, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+    def crop_dark_image(self, initial_crop_parameters, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
 
+    def calculate_harmonic_periods(self, initial_crop_parameters, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+    def run_calculation(self, harm_periods_result, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+    def sort_calculation_result(self, run_calculation_result, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+    def fit_calculation_result(self, sort_calculation_result, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs): raise NotImplementedError()
+
+def create_single_grating_coherence_z_scan_manager(mode=MULTI_THREAD, n_cpus=None):
+    return __SingleGratingCoherenceZScanMultiThread(n_cpus) if mode == MULTI_THREAD else __SingleGratingCoherenceZScanSingleThread()
+
+APPLICATION_NAME = "Single Grating Z Scan"
+
+INITIALIZATION_PARAMETERS_KEY          = APPLICATION_NAME + " Initialization"
 CALCULATE_HARMONIC_PERIODS_CONTEXT_KEY = "Calculate Harmonic Periods"
 RUN_CALCULATION_CONTEXT_KEY            = "Run Calculation"
 SORT_CALCULATION_RESULT_CONTEXT_KEY    = "Sort Calculation Result"
@@ -85,13 +96,31 @@ FIT_CALCULATION_RESULT_CONTEXT_KEY     = "Fit Calculation Result"
 class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
 
     def __init__(self):
-        self.__plotter     = get_registered_plotter_instance()
-        self.__main_logger = get_registered_logger_instance()
-        self.__ini         = get_registered_ini_instance()
+        self.__plotter     = get_registered_plotter_instance(application_name=APPLICATION_NAME)
+        self.__main_logger = get_registered_logger_instance(application_name=APPLICATION_NAME)
+        self.__ini         = get_registered_ini_instance(application_name=APPLICATION_NAME)
 
-    def get_initialization_parameters(self, script_logger_mode, show_fourier=False):
+    def draw_initialization_parameters_widget(self, plotting_properties=PlottingProperties(), **kwargs):
         if self.__plotter.is_active():
-            initialization_parameters = self.__plotter.show_interactive_plot(SGZInputParametersWidget, container_widget=None)
+            add_context_label    = plotting_properties.get_parameter("add_context_label", True)
+            use_unique_id        = plotting_properties.get_parameter("use_unique_id", False)
+
+            unique_id = self.__plotter.register_context_window(INITIALIZATION_PARAMETERS_KEY,
+                                                               context_window=plotting_properties.get_context_widget(),
+                                                               use_unique_id=use_unique_id)
+
+            self.__plotter.push_plot_on_context(INITIALIZATION_PARAMETERS_KEY, SGZInputParametersWidget, unique_id, **kwargs)
+            self.__plotter.draw_context(INITIALIZATION_PARAMETERS_KEY, add_context_label=add_context_label, unique_id=unique_id, **kwargs)
+
+            return self.__plotter.get_plots_of_context(INITIALIZATION_PARAMETERS_KEY, unique_id=unique_id)
+        else:
+            return None
+
+    def get_initialization_parameters(self, plotting_properties=PlottingProperties(), **kwargs):
+        if self.__plotter.is_active():
+            initialization_parameters = self.__plotter.show_interactive_plot(SGZInputParametersDialog,
+                                                                             container_widget=plotting_properties.get_container_widget(),
+                                                                             **kwargs)
         else:
             initialization_parameters = generate_initialization_parameters_sgz(dataFolder         = self.__ini.get_string_from_ini("Files", "data directory"),
                                                                                samplefileName     = self.__ini.get_string_from_ini("Files", "sample file name"),
@@ -101,54 +130,123 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
                                                                                image_per_point    = self.__ini.get_int_from_ini("Parameters", "number of images per step", default=1),
                                                                                strideFile         = self.__ini.get_int_from_ini("Parameters", "stride", default=1),
                                                                                zvec_file          = self.__ini.get_string_from_ini("Parameters", "z distances file"),
-                                                                               pixelSize          = self.__ini.get_float_from_ini("Parameters", "pixel size", default=6.5e-07),
+                                                                               pixelsize          = self.__ini.get_float_from_ini("Parameters", "pixel size", default=6.5e-07),
                                                                                gratingPeriod      = self.__ini.get_float_from_ini("Parameters", "checkerboard grating period", default=4.8e-06),
                                                                                pattern            = self.__ini.get_string_from_ini("Parameters", "pattern", default="Diagonal"),
                                                                                sourceDistanceV    = self.__ini.get_float_from_ini("Parameters", "source distance v", default=-0.73),
                                                                                sourceDistanceH    = self.__ini.get_float_from_ini("Parameters", "source distance h", default=34.0),
                                                                                unFilterSize       = self.__ini.get_int_from_ini("Parameters", "size for uniform filter", default=1),
-                                                                               searchRegion       = self.__ini.get_int_from_ini("Parameters", "size for region for searching", default=1))
+                                                                               searchRegion       = self.__ini.get_int_from_ini("Parameters", "size for region for searching", default=1),
+                                                                               logger=self.__main_logger)
+        return initialization_parameters
 
+    def manager_initialization(self, initialization_parameters, script_logger_mode=LoggerMode.FULL, show_fourier=False):
         initialization_parameters.set_parameter("show_fourier", show_fourier)
 
-        plotter = get_registered_plotter_instance()
-        plotter.register_save_file_prefix(initialization_parameters.get_parameter("saveFileSuf"))
+        self.__plotter.register_save_file_prefix(initialization_parameters.get_parameter("saveFileSuf"))
 
-        if not script_logger_mode == LoggerMode.NONE: stream = open(plotter.get_save_file_prefix() + "_" + common_tools.datetime_now_str() + ".log", "wt")
+        if not script_logger_mode == LoggerMode.NONE: stream = open(self.__plotter.get_save_file_prefix() + "_" + common_tools.datetime_now_str() + ".log", "wt")
         else: stream = None
 
-        register_secondary_logger(stream=stream, logger_mode=script_logger_mode)
+        register_secondary_logger(stream=stream, logger_mode=script_logger_mode, application_name=APPLICATION_NAME)
 
-        self.__script_logger = get_registered_secondary_logger()
+        self.__script_logger = get_registered_secondary_logger(application_name=APPLICATION_NAME)
 
         return initialization_parameters
 
-    def calculate_harmonic_periods(self, initialization_parameters):
-        imgOriginal    = initialization_parameters.get_parameter("img")
-        pattern        = initialization_parameters.get_parameter("pattern")
-        gratingPeriod  = initialization_parameters.get_parameter("gratingPeriod")
-        pixelSize      = initialization_parameters.get_parameter("pixelSize")
+    # %% ==================================================================================================
 
-        self.__plotter.register_context_window(CALCULATE_HARMONIC_PERIODS_CONTEXT_KEY)
+    def draw_crop_initial_image(self, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs):
+        img             = initialization_parameters.get_parameter("img")
+        pixelsize       = initialization_parameters.get_parameter("pixelsize")
+
+        return crop_image.draw_colorbar_crop_image(img=img,
+                                                   pixelsize=pixelsize,
+                                                   plotting_properties=plotting_properties,
+                                                   application_name=APPLICATION_NAME,
+                                                   message="Crop for all Images",
+                                                   **kwargs)
+
+    def crop_initial_image(self, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs):
+        img_original    = initialization_parameters.get_parameter("img")
+        pixelsize       = initialization_parameters.get_parameter("pixelsize")
 
         if self.__plotter.is_active():
-            img, idx4crop, cmap, colorlimit = self.__plotter.show_interactive_plot(ColorbarCropDialogPlot, container_widget=None, img=imgOriginal, pixelsize=[pixelSize, pixelSize], message="Crop for all Images")
+            img, idx4crop, img_size_o, cmap, colorlimit = crop_image.colorbar_crop_image(img=img_original,
+                                                                                         pixelsize=pixelsize,
+                                                                                         plotting_properties=plotting_properties,
+                                                                                         application_name=APPLICATION_NAME,
+                                                                                         message="Crop for all Images",
+                                                                                         **kwargs)
 
-            _, idx4cropDark = self.__plotter.show_interactive_plot(CropDialogPlot,
-                                                                   container_widget=None,
-                                                                   img=imgOriginal,
-                                                                   pixelsize=[pixelSize, pixelSize],
-                                                                   message="Crop for Dark ",
-                                                                   default_idx4crop=[0, 20, 0, 20],
-                                                                   kwargs4graph={'cmap': cmap, 'vmin': colorlimit[0], 'vmax': colorlimit[1]})
-            self.__ini.push()
+            return WavePyData(img_original=img_original,
+                              img=img,
+                              idx4crop=idx4crop,
+                              img_size_o=img_size_o,
+                              cmap=cmap,
+                              colorlimit=colorlimit)
         else:
             idx4crop     = self.__ini.get_list_from_ini("Parameters", "Crop")
-            img          = common_tools.crop_matrix_at_indexes(imgOriginal, idx4crop)
+            img          = common_tools.crop_matrix_at_indexes(img_original, idx4crop)
+
+            return WavePyData(img_original=img_original,
+                              img=img,
+                              idx4crop=idx4crop)
+
+    def draw_crop_dark_image(self, initial_crop_parameters, plotting_properties=PlottingProperties(), **kwargs):
+        img_original = initial_crop_parameters.get_parameter("img_original")
+        cmap         = initial_crop_parameters.get_parameter("cmap")
+        colorlimit   = initial_crop_parameters.get_parameter("colorlimit")
+
+        return crop_image.draw_crop_image(img=img_original,
+                                          plotting_properties=plotting_properties,
+                                          application_name=APPLICATION_NAME,
+                                          message="Crop for Dark",
+                                          default_idx4crop=[0, 20, 0, 20],
+                                          kwargs4graph={'cmap': cmap, 'vmin': colorlimit[0], 'vmax': colorlimit[1]},
+                                          **kwargs)
+
+    def crop_dark_image(self, initial_crop_parameters, plotting_properties=PlottingProperties(), **kwargs):
+        img_original = initial_crop_parameters.get_parameter("img_original")
+        cmap         = initial_crop_parameters.get_parameter("cmap")
+        colorlimit   = initial_crop_parameters.get_parameter("colorlimit")
+
+        if self.__plotter.is_active():
+            _, idx4cropDark, _ = crop_image.crop_image(img=img_original,
+                                                       plotting_properties=plotting_properties,
+                                                       application_name=APPLICATION_NAME,
+                                                       message="Crop for Dark",
+                                                       default_idx4crop=[0, 20, 0, 20],
+                                                       kwargs4graph={'cmap': cmap, 'vmin': colorlimit[0], 'vmax': colorlimit[1]},
+                                                       **kwargs)
+        else:
             idx4cropDark = [0, 20, 0, 20]
 
+        initial_crop_parameters.set_parameter("idx4cropDark", idx4cropDark)
+
+        return initial_crop_parameters
+
+    # %% ==================================================================================================
+
+    def calculate_harmonic_periods(self, initial_crop_parameters, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs):
+        img            = initial_crop_parameters.get_parameter("img",          default_value=initialization_parameters.get_parameter("img"))
+        idx4crop       = initial_crop_parameters.get_parameter("idx4crop",     default_value=self.__ini.get_list_from_ini("Parameters", "Crop"))
+        idx4cropDark   = initial_crop_parameters.get_parameter("idx4cropDark", default_value=[0, 20, 0, 20])
+
+        pattern        = initialization_parameters.get_parameter("pattern")
+        gratingPeriod  = initialization_parameters.get_parameter("gratingPeriod")
+        pixelsize      = initialization_parameters.get_parameter("pixelsize")
+
+        add_context_label = plotting_properties.get_parameter("add_context_label", True)
+        use_unique_id     = plotting_properties.get_parameter("use_unique_id", False)
+
+        unique_id = self.__plotter.register_context_window(CALCULATE_HARMONIC_PERIODS_CONTEXT_KEY,
+                                                           context_window=plotting_properties.get_context_widget(),
+                                                           use_unique_id=use_unique_id)
+
         # Plot Image AFTER crop
-        self.__plotter.push_plot_on_context(CALCULATE_HARMONIC_PERIODS_CONTEXT_KEY, ShowCroppedFigure, img=img, pixelsize=[pixelSize, pixelSize])
+        self.__plotter.push_plot_on_context(CALCULATE_HARMONIC_PERIODS_CONTEXT_KEY, ShowCroppedFigure, unique_id,
+                                            img=img, pixelsize=[pixelsize, pixelsize], allows_saving=False, **kwargs)
 
         self.__main_logger.print_message("Idx for cropping: " + str(idx4crop))
 
@@ -157,11 +255,11 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
         # ==============================================================================
 
         if pattern == PATTERNS[0]:
-            period_harm_Vert = np.int(np.sqrt(2)*pixelSize/gratingPeriod*img.shape[0])
-            period_harm_Horz = np.int(np.sqrt(2)*pixelSize/gratingPeriod*img.shape[1])
+            period_harm_Vert = np.int(np.sqrt(2)*pixelsize/gratingPeriod*img.shape[0])
+            period_harm_Horz = np.int(np.sqrt(2)*pixelsize/gratingPeriod*img.shape[1])
         elif pattern == PATTERNS[1]:
-            period_harm_Vert = np.int(2*pixelSize/gratingPeriod*img.shape[0])
-            period_harm_Horz = np.int(2*pixelSize/gratingPeriod*img.shape[1])
+            period_harm_Vert = np.int(2*pixelsize/gratingPeriod*img.shape[0])
+            period_harm_Horz = np.int(2*pixelsize/gratingPeriod*img.shape[1])
 
         # Obtain harmonic periods from images
 
@@ -171,7 +269,8 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
                                                                        [period_harm_Vert, period_harm_Horz],
                                                                        harmonic_ij=['1', '0'],
                                                                        searchRegion=40,
-                                                                       isFFT=False)
+                                                                       isFFT=False,
+                                                                       logger=self.__main_logger)
 
         self.__main_logger.print_message('Obtain harmonic 01 experimentally')
 
@@ -179,7 +278,8 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
                                                                        [period_harm_Vert, period_harm_Horz],
                                                                        harmonic_ij=['0', '1'],
                                                                        searchRegion=40,
-                                                                       isFFT=False)
+                                                                       isFFT=False,
+                                                                       logger=self.__main_logger)
 
         dataFolder = initialization_parameters.get_parameter("dataFolder")
         startDist = initialization_parameters.get_parameter("startDist")
@@ -199,7 +299,7 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
         if zvec_from == ZVEC_FROM[0]:
             self.__script_logger.print('Step zscan [mm] : {:.4g}'.format(step_z_scan*1e3))
             self.__script_logger.print('Start point zscan [mm] : {:.4g}'.format(startDist*1e3))
-        self.__script_logger.print('Pixel Size [um] : {:.4g}'.format(pixelSize*1e6))
+        self.__script_logger.print('Pixel Size [um] : {:.4g}'.format(pixelsize*1e6))
         self.__script_logger.print('Grating Period [um] : {:.4g}'.format(gratingPeriod*1e6))
         self.__script_logger.print('Grating Pattern : ' + pattern)
         self.__script_logger.print('Crop idxs : ' + str(idx4crop))
@@ -209,14 +309,13 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
         self.__script_logger.print('Uniform Filter Size : {:d}'.format(unFilterSize))
         self.__script_logger.print('Search Region : {:d}'.format(searchRegion))
 
-        self.__draw_context(CALCULATE_HARMONIC_PERIODS_CONTEXT_KEY)
+        self.__plotter.draw_context(CALCULATE_HARMONIC_PERIODS_CONTEXT_KEY, add_context_label=add_context_label, unique_id=unique_id, **kwargs)
 
         return WavePyData(period_harm_Vert=period_harm_Vert, period_harm_Horz=period_harm_Horz, img=img, idx4crop=idx4crop, idx4cropDark=idx4cropDark)
 
+    # %% ==================================================================================================
 
-    def run_calculation(self, harm_periods_result, initialization_parameters):
-        self.__plotter.register_context_window(RUN_CALCULATION_CONTEXT_KEY)
-
+    def run_calculation(self, harm_periods_result, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs):
         period_harm_Vert = harm_periods_result.get_parameter("period_harm_Vert")
         period_harm_Horz = harm_periods_result.get_parameter("period_harm_Horz")
         sample_img       = harm_periods_result.get_parameter("img")
@@ -225,45 +324,62 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
 
         show_fourier    = initialization_parameters.get_parameter("show_fourier", False)
         listOfDataFiles = initialization_parameters.get_parameter("listOfDataFiles")
-        zvec = initialization_parameters.get_parameter("zvec")
+        zvec            = initialization_parameters.get_parameter("zvec")
         sourceDistanceV = initialization_parameters.get_parameter("sourceDistanceV")
         sourceDistanceH = initialization_parameters.get_parameter("sourceDistanceH")
-        unFilterSize = initialization_parameters.get_parameter("unFilterSize")
-        searchRegion = initialization_parameters.get_parameter("searchRegion")
+        unFilterSize    = initialization_parameters.get_parameter("unFilterSize")
+        searchRegion    = initialization_parameters.get_parameter("searchRegion")
+
+        add_context_label = plotting_properties.get_parameter("add_context_label", True)
+        use_unique_id     = plotting_properties.get_parameter("use_unique_id", False)
+
+        unique_id = self.__plotter.register_context_window(RUN_CALCULATION_CONTEXT_KEY,
+                                                           context_window=plotting_properties.get_context_widget(),
+                                                           use_unique_id=use_unique_id)
 
         result = self._get_calculation_result(period_harm_Vert,
-                                           period_harm_Horz,
-                                           idx4crop,
-                                           idx4cropDark,
-                                           listOfDataFiles,
-                                           zvec,
-                                           sourceDistanceV,
-                                           sourceDistanceH,
-                                           unFilterSize,
-                                           searchRegion,
-                                           np.min(zvec))
+                                              period_harm_Horz,
+                                              idx4crop,
+                                              idx4cropDark,
+                                              listOfDataFiles,
+                                              zvec,
+                                              sourceDistanceV,
+                                              sourceDistanceH,
+                                              unFilterSize,
+                                              searchRegion,
+                                              np.min(zvec))
 
-        for i in range(len(result)):
-            imgFFT         = FourierTransform.fft2d(result[i]["img"])
-            harmonicPeriod = result[i]["harmonicPeriod"]
-            image_name     = result[i]["image_name"]
+        if show_fourier:
+            for i in range(len(result)):
+                imgFFT         = FourierTransform.fft2d(result[i]["img"])
+                harmonicPeriod = result[i]["harmonicPeriod"]
+                image_name     = result[i]["image_name"]
 
-            self.__plotter.push_plot_on_context(RUN_CALCULATION_CONTEXT_KEY, HarmonicGridPlot, imgFFT=imgFFT, harmonicPeriod=harmonicPeriod, image_name=image_name, allows_saving=False)
-            self.__plotter.push_plot_on_context(RUN_CALCULATION_CONTEXT_KEY, HarmonicPeakPlot, imgFFT=imgFFT, harmonicPeriod=harmonicPeriod, image_name=image_name, allows_saving=False)
+                self.__plotter.push_plot_on_context(RUN_CALCULATION_CONTEXT_KEY, HarmonicGridPlot, unique_id,
+                                                    imgFFT=imgFFT, harmonicPeriod=harmonicPeriod, image_name=image_name, allows_saving=False, **kwargs)
+                self.__plotter.push_plot_on_context(RUN_CALCULATION_CONTEXT_KEY, HarmonicPeakPlot, unique_id,
+                                                    imgFFT=imgFFT, harmonicPeriod=harmonicPeriod, image_name=image_name, allows_saving=False, **kwargs)
 
-        self.__draw_context(RUN_CALCULATION_CONTEXT_KEY)
+        self.__plotter.draw_context(RUN_CALCULATION_CONTEXT_KEY, add_context_label=add_context_label, unique_id=unique_id, **kwargs)
 
         return WavePyData(res=[res_i["visib_1st_harmonics"] for res_i in result], img=sample_img)
 
-    def sort_calculation_result(self, run_calculation_result, initialization_parameters):
-        self.__plotter.register_context_window(SORT_CALCULATION_RESULT_CONTEXT_KEY)
-
-        pixelSize = initialization_parameters.get_parameter("pixelSize")
-        zvec      = initialization_parameters.get_parameter("zvec")
+    # %% ==================================================================================================
 
+    def sort_calculation_result(self, run_calculation_result, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs):
         img       = run_calculation_result.get_parameter("img")
         res       = run_calculation_result.get_parameter("res")
 
+        pixelsize = initialization_parameters.get_parameter("pixelsize")
+        zvec      = initialization_parameters.get_parameter("zvec")
+
+        add_context_label = plotting_properties.get_parameter("add_context_label", True)
+        use_unique_id     = plotting_properties.get_parameter("use_unique_id", False)
+
+        unique_id = self.__plotter.register_context_window(SORT_CALCULATION_RESULT_CONTEXT_KEY,
+                                                           context_window=plotting_properties.get_context_widget(),
+                                                           use_unique_id=use_unique_id)
+
         contrastV = np.asarray([x[0] for x in res])
         contrastH = np.asarray([x[1] for x in res])
 
@@ -271,41 +387,55 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
         pv = np.asarray([x[3] for x in res])
         ph = np.asarray([x[4] for x in res])
 
-        pattern_period_Vert_z = pixelSize/(pv[:, 0] - p0[:, 0])*img.shape[0]
-        pattern_period_Horz_z = pixelSize/(ph[:, 1] - p0[:, 1])*img.shape[1]
+        pattern_period_Vert_z = pixelsize/(pv[:, 0] - p0[:, 0])*img.shape[0]
+        pattern_period_Horz_z = pixelsize/(ph[:, 1] - p0[:, 1])*img.shape[1]
 
         self.__plotter.save_csv_file(np.c_[zvec.T, contrastV.T, contrastH.T, pattern_period_Vert_z.T, pattern_period_Horz_z.T],
                                      headerList=['z [m]', 'Vert Contrast', 'Horz Contrast', 'Vert Period [m]', 'Horz Period [m]'])
 
-        self.__plotter.push_plot_on_context(SORT_CALCULATION_RESULT_CONTEXT_KEY, VisibilityPlot, zvec=zvec, contrastV=contrastV, contrastH=contrastH)
+        self.__plotter.push_plot_on_context(SORT_CALCULATION_RESULT_CONTEXT_KEY, VisibilityPlot, unique_id,
+                                            zvec=zvec, contrastV=contrastV, contrastH=contrastH, **kwargs)
 
-        self.__draw_context(SORT_CALCULATION_RESULT_CONTEXT_KEY)
+        self.__plotter.draw_context(SORT_CALCULATION_RESULT_CONTEXT_KEY, add_context_label=add_context_label, unique_id=unique_id, **kwargs)
 
         return WavePyData(pattern_period_Vert_z=pattern_period_Vert_z, pattern_period_Horz_z=pattern_period_Horz_z, contrastV=contrastV, contrastH=contrastH)
 
-    def fit_calculation_result(self, sort_calculation_result, initialization_parameters):
-        self.__plotter.register_context_window(FIT_CALCULATION_RESULT_CONTEXT_KEY)
-
-        zvec                   = initialization_parameters.get_parameter("zvec")
+    # %% ==================================================================================================
 
+    def fit_calculation_result(self, sort_calculation_result, initialization_parameters, plotting_properties=PlottingProperties(), **kwargs):
         pattern_period_Vert_z  = sort_calculation_result.get_parameter("pattern_period_Vert_z")
         contrastV              = sort_calculation_result.get_parameter("contrastV")
         pattern_period_Horz_z  = sort_calculation_result.get_parameter("pattern_period_Horz_z")
         contrastH              = sort_calculation_result.get_parameter("contrastH")
 
+        zvec                   = initialization_parameters.get_parameter("zvec")
+
+        add_context_label = plotting_properties.get_parameter("add_context_label", True)
+        use_unique_id     = plotting_properties.get_parameter("use_unique_id", False)
+
+        unique_id = self.__plotter.register_context_window(FIT_CALCULATION_RESULT_CONTEXT_KEY,
+                                                           context_window=plotting_properties.get_context_widget(),
+                                                           use_unique_id=use_unique_id)
+
         sourceDistance_from_fit_V, patternPeriodFromData_V = self.__fit_period_vs_z(zvec,
                                                                                     pattern_period_Vert_z,
                                                                                     contrastV,
                                                                                     direction='Vertical',
-                                                                                    threshold=0.002)
+                                                                                    threshold=0.002,
+                                                                                    context_key=FIT_CALCULATION_RESULT_CONTEXT_KEY,
+                                                                                    unique_id=unique_id,
+                                                                                    **kwargs)
 
         sourceDistance_from_fit_H, patternPeriodFromData_H = self.__fit_period_vs_z(zvec,
                                                                                     pattern_period_Horz_z,
                                                                                     contrastH,
                                                                                     direction='Horizontal',
-                                                                                    threshold=0.0005)
+                                                                                    threshold=0.0005,
+                                                                                    context_key=FIT_CALCULATION_RESULT_CONTEXT_KEY,
+                                                                                    unique_id=unique_id,
+                                                                                    **kwargs)
 
-        self.__draw_context(FIT_CALCULATION_RESULT_CONTEXT_KEY)
+        self.__plotter.draw_context(FIT_CALCULATION_RESULT_CONTEXT_KEY, add_context_label=add_context_label, unique_id=unique_id, **kwargs)
 
         return WavePyData(sourceDistance_from_fit_V=sourceDistance_from_fit_V,
                           patternPeriodFromData_V=patternPeriodFromData_V,
@@ -328,7 +458,7 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
                                 searchRegion,
                                 min_zvec): raise NotImplementedError()
 
-    def __fit_period_vs_z(self, zvec, pattern_period_z, contrast, direction, threshold=0.005, context_key=FIT_CALCULATION_RESULT_CONTEXT_KEY):
+    def __fit_period_vs_z(self, zvec, pattern_period_z, contrast, direction, threshold=0.005, context_key=FIT_CALCULATION_RESULT_CONTEXT_KEY, unique_id=None, **kwargs):
         args_for_NOfit = np.argwhere(contrast < threshold).flatten()
         args_for_fit = np.argwhere(contrast >= threshold).flatten()
 
@@ -343,7 +473,7 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
 
         output_data = WavePyData()
 
-        self.__plotter.push_plot_on_context(context_key, FitPeriodVsZPlot,
+        self.__plotter.push_plot_on_context(context_key, FitPeriodVsZPlot, unique_id,
                                             zvec=zvec,
                                             args_for_NOfit=args_for_NOfit,
                                             args_for_fit=args_for_fit,
@@ -352,14 +482,12 @@ class __SingleGratingCoherenceZScan(SingleGratingCoherenceZScanFacade):
                                             ls1=ls1,
                                             lc2=lc2,
                                             direction=direction,
-                                            output_data=output_data)
+                                            output_data=output_data,
+                                            **kwargs)
 
 
         return output_data.get_parameter("sourceDistance"), output_data.get_parameter("patternPeriodFromData")
 
-    def __draw_context(self, context_key):
-        self.__plotter.draw_context_on_widget(context_key, container_widget=self.__plotter.get_context_container_widget(context_key))
-
 #=========================================================================================
 # MULTI-THREADING SECTION
 #=========================================================================================
@@ -378,7 +506,9 @@ def _run_calculation(parameters):
         searchRegion, \
         unFilterSize = parameters
 
-        get_registered_logger_instance().print_message("loop " + str(i) + ": " + data_file_i)
+        # python3.8 do not share the same environment, so the Singleton is not active
+        try: get_registered_logger_instance(APPLICATION_NAME).print_message("loop " + str(i) + ": " + data_file_i)
+        except: print("loop " + str(i) + ": " + data_file_i)
 
         img = read_tiff(data_file_i)
 
@@ -441,6 +571,16 @@ from multiprocessing import Pool, cpu_count
 import time
 
 class __SingleGratingCoherenceZScanMultiThread(__SingleGratingCoherenceZScan):
+    def __init__(self, n_cpus=None):
+        super().__init__()
+        available_cpus = cpu_count()
+        if not n_cpus is None and n_cpus > 0:
+            if n_cpus > available_cpus - 1: raise ValueError("Max number of CPUs available = " + str(available_cpus-1))
+            self.__n_cpus = n_cpus
+        else:
+            self.__n_cpus = available_cpus - 2
+            if self.__n_cpus < 2: raise ValueError("Auto Nr. CPUs available < 2: Multi-Thread mode not possible")
+
     def _get_calculation_result(self,
                                 period_harm_Vert,
                                 period_harm_Horz,
@@ -454,11 +594,10 @@ class __SingleGratingCoherenceZScanMultiThread(__SingleGratingCoherenceZScan):
                                 searchRegion,
                                 min_zvec):
 
-        ncpus = cpu_count() - 2
         tzero = time.time()
-        pool = Pool(ncpus)
+        pool = Pool(self.__n_cpus)
 
-        get_registered_logger_instance().print_message("%d cpu's available" % ncpus)
+        get_registered_logger_instance(APPLICATION_NAME).print_message("%d cpu used for this calculation" % self.__n_cpus)
 
         parameters = []
         for i in range(len(listOfDataFiles)): parameters.append([i,
@@ -476,6 +615,6 @@ class __SingleGratingCoherenceZScanMultiThread(__SingleGratingCoherenceZScan):
         res = pool.map(_run_calculation, parameters)
         pool.close()
 
-        get_registered_logger_instance().print_message("Time spent: {0:.3f} s".format(time.time() - tzero))
+        get_registered_logger_instance(APPLICATION_NAME).print_message("Time spent: {0:.3f} s".format(time.time() - tzero))
 
         return res
