@@ -52,6 +52,7 @@ from PyQt5.QtCore import Qt
 from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 
 class WavePyGenericWidget(object):
+    def __init__(self, application_name=None): self._application_name = application_name
     def build_widget(self, **kwargs): raise NotImplementedError()
 
 class FigureToSave():
@@ -64,8 +65,9 @@ class FigureToSave():
 
 
 class WavePyWidget(QWidget, WavePyGenericWidget):
-    def __init__(self, parent=None):
-        super(WavePyWidget, self).__init__(parent=parent)
+    def __init__(self, parent=None, application_name=None, **kwargs):
+        QWidget.__init__(self, parent=parent)
+        WavePyGenericWidget.__init__(self, application_name=application_name)
 
         self.__allows_saving = True
 
@@ -83,8 +85,14 @@ class WavePyWidget(QWidget, WavePyGenericWidget):
 
         self.append_mpl_figure_to_save(canvas.figure)
 
-        self.setFixedWidth(canvas.get_width_height()[0]*1.1)
-        self.setFixedHeight(canvas.get_width_height()[1]*1.1)
+        try:    widget_width = kwargs["widget_width"]
+        except: widget_width = canvas.get_width_height()[0]*1.1
+        try:    widget_height = kwargs["widget_height"]
+        except: widget_height = canvas.get_width_height()[1]*1.1
+
+        self.setFixedWidth(widget_width)
+        self.setFixedHeight(widget_height)
+
         layout.setStretchFactor(canvas, 1)
         layout.addWidget(canvas)
 
@@ -94,7 +102,7 @@ class WavePyWidget(QWidget, WavePyGenericWidget):
         if not hasattr(self, "__figures_to_save") or self.__figures_to_save is None: self.__figures_to_save = []
         self.__figures_to_save.append(FigureToSave(figure=figure,
                                                    figure_file_name=figure_file_name if not common_tools.is_empty_string(figure_file_name) else \
-                                                       common_tools.get_unique_filename(get_registered_plotter_instance().get_save_file_prefix(), "png")))
+                                                       common_tools.get_unique_filename(get_registered_plotter_instance(application_name=self._application_name).get_save_file_prefix(), "png")))
 
     def build_mpl_figure(self, **kwargs): raise NotImplementedError()
 
@@ -107,8 +115,9 @@ class WavePyWidget(QWidget, WavePyGenericWidget):
 
 class WavePyInteractiveWidget(QDialog, WavePyGenericWidget):
 
-    def __init__(self, parent, message, title):
-        super(QDialog, self).__init__(parent)
+    def __init__(self, parent, message, title, application_name=None, **kwargs):
+        QDialog.__init__(self, parent)
+        WavePyGenericWidget.__init__(self, application_name=application_name)
 
         self.setWindowTitle(message)
         self.setModal(True)
@@ -157,15 +166,16 @@ class WavePyInteractiveWidget(QDialog, WavePyGenericWidget):
 class PlotterFacade:
     def is_active(self): raise NotImplementedError()
     def is_saving(self): raise NotImplementedError()
-    def register_context_window(self, context_key, context_window=None): raise NotImplementedError()
+    def register_context_window(self, context_key, context_window=None, use_unique_id=False): raise NotImplementedError()
     def register_save_file_prefix(self, save_file_prefix): raise NotImplementedError()
-    def push_plot_on_context(self, context_key, widget_class, **kwargs): raise NotImplementedError()
-    def get_plots_of_context(self, context_key): raise NotImplementedError()
-    def get_context_container_widget(self, context_key): raise  NotImplementedError()
+    def push_plot_on_context(self, context_key, widget_class, unique_id=None, **kwargs): raise NotImplementedError()
+    def get_plots_of_context(self, context_key, unique_id=None): raise NotImplementedError()
+    def get_context_container_widget(self, context_key, unique_id=None): raise  NotImplementedError()
     def get_save_file_prefix(self): raise NotImplementedError()
-    def draw_context_on_widget(self, context_key, container_widget): raise NotImplementedError()
+    def draw_context_on_widget(self, context_key, container_widget, add_context_label=True, unique_id=None, **kwargs): raise NotImplementedError()
+    def draw_context(self, context_key, add_context_label=True, unique_id=None, **kwargs): raise NotImplementedError
     def show_interactive_plot(self, widget_class, container_widget, **kwargs): raise NotImplementedError()
-    def show_context_window(self, context_key): raise NotImplementedError()
+    def show_context_window(self, context_key, unique_id=None): raise NotImplementedError()
     def save_sdf_file(self, array, pixelsize, file_prefix, file_suffix, extraHeader): raise NotImplementedError()
     def save_csv_file(self, array_list, file_prefix, file_suffix, headerList, comments): raise NotImplementedError()
 
@@ -182,7 +192,10 @@ class PlotterMode:
         if plotter_mode==cls.SAVE_ONLY: return "Save Only" 
         if plotter_mode==cls.NONE: return "None" 
 
-class __AbstractPlotter(PlotterFacade):
+class _AbstractPlotter(PlotterFacade):
+
+    def __init__(self, application_name=None):
+        self._application_name = application_name
 
     @classmethod
     def _save_images(cls, plot_widget_instance, **kwargs):
@@ -192,11 +205,11 @@ class __AbstractPlotter(PlotterFacade):
             for figure_to_save in figures_to_save: figure_to_save.save_figure(**kwargs)
 
     @classmethod
-    def _build_plot(cls, widget_class, **kwargs):
+    def _build_plot(cls, widget_class, application_name, **kwargs):
         if not issubclass(widget_class, WavePyWidget): raise ValueError("Widget class is not a WavePyWidget")
 
         try:
-            plot_widget_instance = widget_class()
+            plot_widget_instance = widget_class(parent=None, application_name=application_name, **kwargs)
             plot_widget_instance.build_widget(**kwargs)
 
             return plot_widget_instance
@@ -208,49 +221,67 @@ class __AbstractPlotter(PlotterFacade):
     def get_save_file_prefix(self): return self.__save_file_prefix
 
     def _get_file_name(self, file_prefix=None, file_suffix="", extension=""):
-        return common_tools.get_unique_filename(self.get_save_file_prefix() if file_prefix is None else file_prefix + file_suffix, extension)
+        return common_tools.get_unique_filename(str(self.get_save_file_prefix() if file_prefix is None else file_prefix) + file_suffix, extension)
 
     def save_sdf_file(self, array, pixelsize=[1, 1], file_prefix=None, file_suffix="", extraHeader={}):
         file_name = self._get_file_name(file_prefix, file_suffix, "sdf")
-        plot_tools.save_sdf_file(array, pixelsize, file_name, extraHeader)
+        plot_tools.save_sdf_file(array, pixelsize, file_name, extraHeader, self._application_name)
 
         return file_name
 
     def save_csv_file(self, array_list, file_prefix=None, file_suffix="", headerList=[], comments=""):
         file_name = self._get_file_name(file_prefix, file_suffix, "csv")
-        plot_tools.save_csv_file(array_list, file_name, headerList, comments)
+        plot_tools.save_csv_file(array_list, file_name, headerList, comments, self._application_name)
 
         return file_name
 
+    def draw_context(self, context_key, add_context_label=True, unique_id=None, **kwargs):
+        self.draw_context_on_widget(context_key, self.get_context_container_widget(context_key, unique_id), add_context_label, unique_id, **kwargs)
+
 from wavepy2.util.plot.plot_tools import DefaultMainWindow
 
-class __AbstractActivePlotter(__AbstractPlotter):
-    def __init__(self):
+class _AbstractActivePlotter(_AbstractPlotter):
+    def __init__(self, application_name=None):
+        _AbstractPlotter.__init__(self, application_name=application_name)
         self.__plot_registry = {}
         self.__context_window_registry = {}
 
     def is_active(self): return True
 
-    def _register_plot(self, context_key, plot_widget):
+    def _register_plot(self, context_key, plot_widget, unique_id=None):
+        if not unique_id is None: context_key += "_" + unique_id
+
         if context_key in self.__plot_registry and not self.__plot_registry[context_key] is None:
             self.__plot_registry[context_key].append(plot_widget)
         else:
             self.__plot_registry[context_key] = [plot_widget]
 
-    def register_context_window(self, context_key, context_window=None):
+    def register_context_window(self, context_key, context_window=None, use_unique_id=False):
         if context_window is None: context_window = DefaultMainWindow(context_key)
-        self.__context_window_registry[context_key] = context_window
+        if use_unique_id:
+            unique_id = str(id(context_window))
+            self.__context_window_registry[context_key + "_" + unique_id] = context_window
+            return unique_id
+        else:
+            self.__context_window_registry[context_key] = context_window
+            return None
 
-    def get_plots_of_context(self, context_key):
+    def get_plots_of_context(self, context_key, unique_id=None):
+        if not unique_id is None: context_key += "_" + unique_id
         if context_key in self.__plot_registry: return self.__plot_registry[context_key]
         else: return None
 
-    def get_context_container_widget(self, context_key):
+    def get_context_container_widget(self, context_key, unique_id=None):
+        if not unique_id is None: context_key += "_" + unique_id
+
         if context_key in self.__context_window_registry: return self.__context_window_registry[context_key].get_container_widget()
         else: return None
 
-    def draw_context_on_widget(self, context_key, container_widget):
-        main_box = plot_tools.widgetBox(container_widget, context_key, orientation="horizontal")
+    def draw_context_on_widget(self, context_key, container_widget, add_context_label=True, unique_id=None, **kwargs):
+        if not unique_id is None: context_key += "_" + unique_id
+        container_widget.setStyleSheet(plot_tools.stylesheet_string)
+
+        main_box = plot_tools.widgetBox(container_widget, context_key if add_context_label else "", orientation="horizontal")
         main_box.layout().setAlignment(Qt.AlignCenter)
         tab_widget = plot_tools.tabWidget(main_box)
 
@@ -258,17 +289,34 @@ class __AbstractActivePlotter(__AbstractPlotter):
         heights = []
 
         if context_key in self.__plot_registry:
-            for plot_widget_instance in self.__plot_registry[context_key]:
+            plot_widget_instances = self.__plot_registry[context_key]
+
+            for plot_widget_instance in plot_widget_instances:
                 tab = plot_tools.createTabPage(tab_widget, plot_widget_instance.get_plot_tab_name())
                 tab.layout().setAlignment(Qt.AlignCenter)
                 tab.layout().addWidget(plot_widget_instance)
                 widths.append(plot_widget_instance.width())
                 heights.append(plot_widget_instance.height())
+        else:
+            label = plot_tools.widgetLabel(tab_widget, "\n\n\n\n\n        Nothing to Display")
+            label.setStyleSheet("font: 24pt")
+            widths.append(500)
+            heights.append(370)
 
-        tab_widget.setFixedWidth(max(widths)*1.05)
-        tab_widget.setFixedHeight(max(heights)*1.05)
-        container_widget.setFixedWidth(max(widths)*1.07)
-        container_widget.setFixedHeight(max(heights)*1.1)
+        try:    tab_widget_width = kwargs["tab_widget_width"]
+        except: tab_widget_width = max(widths) + 20
+        try:    tab_widget_height = kwargs["tab_widget_height"]
+        except: tab_widget_height = max(heights) + 35
+
+        try:    container_widget_width  = kwargs["container_widget_width"]
+        except: container_widget_width  = tab_widget_width + 25
+        try:    container_widget_height = kwargs["container_widget_height"]
+        except: container_widget_height = tab_widget_height + 55
+
+        tab_widget.setFixedWidth(tab_widget_width)
+        tab_widget.setFixedHeight(tab_widget_height)
+        container_widget.setFixedWidth(container_widget_width)
+        container_widget.setFixedHeight(container_widget_height)
 
         container_widget.update()
 
@@ -276,86 +324,89 @@ class __AbstractActivePlotter(__AbstractPlotter):
         if not issubclass(widget_class, WavePyInteractiveWidget): raise ValueError("Widget class is not a WavePyWidget")
 
         try:
-            interactive_widget_instance = widget_class(parent=container_widget)
+            interactive_widget_instance = widget_class(parent=container_widget, application_name=self._application_name, **kwargs)
             interactive_widget_instance.build_widget(**kwargs)
         except Exception as e:
             raise ValueError("Plot Widget can't be created: " + str(e))
 
         return widget_class.get_output(interactive_widget_instance)
 
-    def show_context_window(self, context_key):
+    def show_context_window(self, context_key, unique_id=None):
+        if not unique_id is None: context_key += "_" + unique_id
         if context_key in self.__context_window_registry: self.__context_window_registry[context_key].show()
         else: pass
 
-class __FullPlotter(__AbstractActivePlotter):
+class __FullPlotter(_AbstractActivePlotter):
+    def __init__(self, application_name=None): _AbstractActivePlotter.__init__(self, application_name=application_name)
     def is_saving(self): return True
-    def push_plot_on_context(self, context_key, widget_class, **kwargs):
-        plot_widget_instance = self._build_plot(widget_class, **kwargs)
-        self._register_plot(context_key, plot_widget_instance)
+    def push_plot_on_context(self, context_key, widget_class, unique_id=None, **kwargs):
+        plot_widget_instance = self._build_plot(widget_class=widget_class, application_name=self._application_name, **kwargs)
+        self._register_plot(context_key, plot_widget_instance, unique_id)
         self._save_images(plot_widget_instance, **kwargs)
 
-class __DisplayOnlyPlotter(__AbstractActivePlotter):
+class __DisplayOnlyPlotter(_AbstractActivePlotter):
+    def __init__(self, application_name=None): _AbstractActivePlotter.__init__(self, application_name=application_name)
     def is_saving(self): return False
-    def push_plot_on_context(self, context_key, widget_class, **kwargs): self._register_plot(context_key, self._build_plot(widget_class, **kwargs))
+    def push_plot_on_context(self, context_key, widget_class, unique_id=None, **kwargs): self._register_plot(context_key, self._build_plot(widget_class=widget_class, application_name=self._application_name, **kwargs), unique_id)
     def save_sdf_file(self, array, pixelsize=[1, 1], file_prefix=None, file_suffix="", extraHeader={}): return self._get_file_name(file_prefix, file_suffix, "sdf")
     def save_csv_file(self, array_list, file_prefix=None, file_suffix="", headerList=[], comments=""): return self._get_file_name(file_prefix, file_suffix, "csv")
 
-class __SaveOnlyPlotter(__AbstractActivePlotter):
+class __SaveOnlyPlotter(_AbstractActivePlotter):
+    def __init__(self, application_name=None): _AbstractActivePlotter.__init__(self, application_name=application_name)
     def is_active(self): return False
     def is_saving(self): return True
-    def register_context_window(self, context_key, context_window=None): pass
-    def push_plot_on_context(self, context_key, widget_class, **kwargs): self._save_images(self._build_plot(widget_class, **kwargs))
-    def get_context_container_widget(self, context_key): return None
-    def get_plots_of_context(self, context_key): pass
-    def draw_context_on_widget(self, context_key, container_widget): pass
+    def register_context_window(self, context_key, context_window=None, use_unique_id=False): pass
+    def push_plot_on_context(self, context_key, widget_class, unique_id=None, **kwargs): self._save_images(self._build_plot(widget_class=widget_class, application_name=self._application_name, **kwargs))
+    def get_context_container_widget(self, context_key, unique_id=None): return None
+    def get_plots_of_context(self, context_key, unique_id=None): pass
+    def draw_context_on_widget(self, context_key, container_widget, add_context_label=True, unique_id=None, **kwargs): pass
     def show_interactive_plot(self, widget_class, container_widget, **kwargs): pass
-    def show_context_window(self, context_key): pass
+    def show_context_window(self, context_key, unique_id=None): pass
 
-class __NullPlotter(__AbstractPlotter):
+class __NullPlotter(_AbstractPlotter):
+    def __init__(self, application_name=None): _AbstractActivePlotter.__init__(self, application_name=application_name)
     def is_active(self): return False
     def is_saving(self): return False
-    def register_context_window(self, context_key, context_window=None): pass
-    def push_plot_on_context(self, context_key, widget_class, **kwargs): self._build_plot(widget_class, **kwargs) # necessary for some operations
-    def get_context_container_widget(self, context_key): return None
-    def get_plots_of_context(self, context_key): pass
-    def draw_context_on_widget(self, context_key, container_widget): pass
+    def register_context_window(self, context_key, context_window=None, use_unique_id=False): pass
+    def push_plot_on_context(self, context_key, widget_class, unique_id=None, **kwargs): self._build_plot(widget_class=widget_class, application_name=self._application_name, **kwargs) # necessary for some operations
+    def get_context_container_widget(self, context_key, unique_id=None): return None
+    def get_plots_of_context(self, context_key, unique_id=None): pass
+    def draw_context_on_widget(self, context_key, container_widget, add_context_label=True, unique_id=None, **kwargs): pass
     def show_interactive_plot(self, widget_class, container_widget, **kwargs): pass
-    def show_context_window(self, context_key): pass
+    def show_context_window(self, context_key, unique_id=None): pass
     def save_sdf_file(self, array, pixelsize=[1, 1], file_prefix=None, file_suffix="", extraHeader={}): return self._get_file_name(file_prefix, file_suffix, "sdf")
     def save_csv_file(self, array_list, file_prefix=None, file_suffix="", headerList=[], comments=""): return self._get_file_name(file_prefix, file_suffix, "csv")
 
+from wavepy2.util.common.common_tools import GenericRegistry
+
 @Singleton
-class __PlotterRegistry:
+class __PlotterRegistry(GenericRegistry):
 
     def __init__(self):
-        self.__plotter_instance = None
+        GenericRegistry.__init__(self, registry_name="Plotter")
 
     @synchronized_method
-    def register_plotter(self, plotter_facade_instance = None):
-        if plotter_facade_instance is None: raise ValueError("Plotter Instance is None")
-        if not isinstance(plotter_facade_instance, PlotterFacade): raise ValueError("Plotter Instance do not implement Plotter Facade")
-
-        if self.__plotter_instance is None: self.__plotter_instance = plotter_facade_instance
-        else: raise ValueError("Plotter Instance already initialized")
+    def register_plotter(self, plotter_facade_instance, application_name=None):
+        super().register_instance(plotter_facade_instance, application_name)
 
     @synchronized_method
-    def reset(self):
-        self.__plotter_instance = None
+    def reset(self, application_name=None):
+        super().reset(application_name)
 
-    def get_plotter_instance(self):
-        return self.__plotter_instance
+    def get_plotter_instance(self, application_name=None):
+        return super().get_instance(application_name)
 
 # -----------------------------------------------------
 # Factory Methods
 
-def register_plotter_instance(plotter_mode=PlotterMode.FULL, reset=False):
+def register_plotter_instance(plotter_mode=PlotterMode.FULL, reset=False, application_name=None):
     if reset: __PlotterRegistry.Instance().reset()
 
-    if plotter_mode   == PlotterMode.FULL:         __PlotterRegistry.Instance().register_plotter(__FullPlotter())
-    elif plotter_mode == PlotterMode.DISPLAY_ONLY: __PlotterRegistry.Instance().register_plotter(__DisplayOnlyPlotter())
-    elif plotter_mode == PlotterMode.SAVE_ONLY:    __PlotterRegistry.Instance().register_plotter(__SaveOnlyPlotter())
-    elif plotter_mode == PlotterMode.NONE:         __PlotterRegistry.Instance().register_plotter(__NullPlotter())
+    if plotter_mode   == PlotterMode.FULL:         __PlotterRegistry.Instance().register_plotter(__FullPlotter(application_name), application_name)
+    elif plotter_mode == PlotterMode.DISPLAY_ONLY: __PlotterRegistry.Instance().register_plotter(__DisplayOnlyPlotter(application_name), application_name)
+    elif plotter_mode == PlotterMode.SAVE_ONLY:    __PlotterRegistry.Instance().register_plotter(__SaveOnlyPlotter(application_name), application_name)
+    elif plotter_mode == PlotterMode.NONE:         __PlotterRegistry.Instance().register_plotter(__NullPlotter(application_name), application_name)
 
-def get_registered_plotter_instance():
-    return __PlotterRegistry.Instance().get_plotter_instance()
+def get_registered_plotter_instance(application_name=None):
+    return __PlotterRegistry.Instance().get_plotter_instance(application_name)
 
